///////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2004 Intel Corporation 
// All rights reserved. 
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met: 
//
// * Redistributions of source code must retain the above copyright notice, 
// this list of conditions and the following disclaimer. 
// * Redistributions in binary form must reproduce the above copyright notice, 
// this list of conditions and the following disclaimer in the documentation 
// and/or other materials provided with the distribution. 
// * Neither name of Intel Corporation nor the names of its contributors 
// may be used to endorse or promote products derived from this software 
// without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
// OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ipp.h"

#include "mask.h"
#include "Image.h"

Mask::Mask() : Image8u() {}
Mask::Mask(IppiSize size, int channelCount) : Image8u(size, 1) {}
Mask::Mask(int w, int h, int channelCount) : Image8u(w, h, 1) {}
Mask::Mask(const Image8u* pImage) : Image8u()
{
	if (pImage->GetChannelCount() == 1)
	{
		CopySettings(pImage);
		CopyFrom(pImage->GetConstData(), pImage->GetStride());
	}
	else
	{
		InitAlloc(pImage->GetSize(), 1);
	}
}

Mask::~Mask() {}

int Mask::Compare_(const Image8u* pSrc1, const Image8u* pSrc2, IppCmpOp op)
{
	int st;
	if (st = pSrc1->CheckCompat(pSrc2)) return st;

	int nChannels = pSrc1->GetChannelCount();

	IppiSize size = GetOverlap(pSrc1->GetOverlap(pSrc2));

	if (nChannels == 1)
		st = (int)ippiCompare_8u_C1R(
			pSrc1->GetConstData(), pSrc1->GetStride(),
			pSrc2->GetConstData(), pSrc2->GetStride(),
			GetData(), GetStride(),
			size, op);
	else if (nChannels == 3)
		st = (int)ippiCompare_8u_C3R(
			pSrc1->GetConstData(), pSrc1->GetStride(),
			pSrc2->GetConstData(), pSrc2->GetStride(),
			GetData(), GetStride(),
			size, op);
	else if (nChannels == 4)
		st = (int)ippiCompare_8u_C4R(
			pSrc1->GetConstData(), pSrc1->GetStride(),
			pSrc2->GetConstData(), pSrc2->GetStride(),
			GetData(), GetStride(),
			size, op);
	else st = -1;
	
	return st;
}

int Mask::CreateByCompare_GT(const Image8u* pSrc1, const Image8u* pSrc2)
{
	return Compare_(pSrc1, pSrc2, ippCmpGreater);
}
int Mask::CreateByCompare_GTE(const Image8u* pSrc1, const Image8u* pSrc2)
{
	return Compare_(pSrc1, pSrc2, ippCmpGreaterEq);
}

int Mask::CreateByCompare_EQ(const Image8u* pSrc1, const Image8u* pSrc2)
{
	return Compare_(pSrc1, pSrc2, ippCmpEq);
}

int Mask::Compare_(const Image8u* pSrc, Ipp8u threshold, IppCmpOp op)
{
	Ipp8u pThresholds[4] = { threshold, threshold, threshold, threshold };
	if (pSrc->GetChannelCount() != 1)
		return Compare_(pSrc, pThresholds, op);

	int st;

	IppiSize size = GetOverlap(pSrc);

	st = (int)ippiCompareC_8u_C1R( pSrc->GetConstData(), pSrc->GetStride(),
		threshold, GetData(), GetStride(), size, op);
	return st;
}

int Mask::CreateByCompare_GT(const Image8u* pSrc, Ipp8u threshold)
{
	return Compare_(pSrc, threshold, ippCmpGreater);
}
int Mask::CreateByCompare_GTE(const Image8u* pSrc, Ipp8u threshold)
{
	return Compare_(pSrc, threshold, ippCmpGreaterEq);
}
int Mask::CreateByCompare_LT(const Image8u* pSrc, Ipp8u threshold)
{
	return Compare_(pSrc, threshold, ippCmpLess);
}
int Mask::CreateByCompare_EQ(const Image8u* pSrc, Ipp8u threshold)
{
	return Compare_(pSrc, threshold, ippCmpEq);
}

int Mask::Compare_(const Image8u* pSrc, Ipp8u* pThresholds, IppCmpOp op)
{
	if (pSrc->GetChannelCount() == 1)
		return Compare_(pSrc, *pThresholds, op);

	int st;

	IppiSize size = GetOverlap(pSrc);

	if (pSrc->GetChannelCount() == 3)
		st = (int)ippiCompareC_8u_C3R( pSrc->GetConstData(), pSrc->GetStride(),
		pThresholds, GetData(), GetStride(), size, op);
	else if (pSrc->GetChannelCount() == 4)
		st = (int)ippiCompareC_8u_C4R( pSrc->GetConstData(), pSrc->GetStride(),
		pThresholds, GetData(), GetStride(), size, op);
	else st = -1;

	return st;
}

int Mask::CreateByCompare_GT(const Image8u* pSrc, Ipp8u* pThresholds)
{
	return Compare_(pSrc, pThresholds, ippCmpGreater);
}
int Mask::CreateByCompare_GTE(const Image8u* pSrc, Ipp8u* pThresholds)
{
	return Compare_(pSrc, pThresholds, ippCmpGreaterEq);
}
int Mask::CreateByCompare_LT(const Image8u* pSrc, Ipp8u* pThresholds)
{
	return Compare_(pSrc, pThresholds, ippCmpLess);
}
int Mask::CreateByCompare_EQ(const Image8u* pSrc, Ipp8u* pThresholds)
{
	return Compare_(pSrc, pThresholds, ippCmpEq);
}

int Mask::CopyWithMask(const Image8u* pSrc, Image8u* pDst)
{
	IppiSize size = GetOverlap(pSrc->GetOverlap(pDst));

	IppiPoint p = {0,0};
	IppiRect rect = {0,0, size.width, size.height};
	return CopyWithMask_R(pSrc, rect, pDst, p);
}

int Mask::CopyWithMask_R(const Image8u* pSrc, IppiRect srcROI,
		   Image8u* pDst, IppiPoint dstOffset)
{
	int st;
	if (st = pDst->CheckCompat(pSrc)) return st;

	IppiRect rect = GetOverlap(pSrc->GetOverlap(pDst,dstOffset),srcROI);
	IppiSize size = {rect.width, rect.height};
	if (pSrc->GetChannelCount()==1)
		st = (int)ippiCopy_8u_C1MR(
						pSrc->GetConstData(rect.x, rect.y), pSrc->GetStride(),
						pDst->GetData(rect.x, rect.y), pDst->GetStride(), size,
						GetData(rect.x, rect.y), GetStride());
	else if (pSrc->GetChannelCount()==3)
		st = (int)ippiCopy_8u_C3MR(
						pSrc->GetConstData(rect.x, rect.y), pSrc->GetStride(),
						pDst->GetData(rect.x, rect.y), pDst->GetStride(), size,
						GetData(rect.x, rect.y), GetStride());
	else if (pSrc->GetChannelCount()==4)
		st = (int)ippiCopy_8u_C4MR(
						pSrc->GetConstData(rect.x, rect.y), pSrc->GetStride(),
						pDst->GetData(rect.x, rect.y), pDst->GetStride(), size,
						GetData(rect.x, rect.y), GetStride());
	else st = -1;
	return st;
}
